"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const getElementTagName_1 = __importDefault(require("./getElementTagName"));
const executeScript_1 = __importDefault(require("./executeScript"));
const constants_1 = require("../constants");
const utils_1 = require("../utils");
const SELECT_SCRIPT = 'return (function select (elem) { elem.selected = true }).apply(null, arguments)';
async function elementClick({ elementId }) {
    const page = this.getPageHandle();
    const elementHandle = await this.elementStore.get(elementId);
    if (!elementHandle) {
        throw utils_1.getStaleElementError(elementId);
    }
    const tagName = await getElementTagName_1.default.call(this, { elementId });
    if (tagName === 'option') {
        return executeScript_1.default.call(this, {
            script: SELECT_SCRIPT,
            args: [{ [constants_1.ELEMENT_KEY]: elementId }]
        });
    }
    return new Promise((resolve, reject) => {
        const dialogHandler = () => resolve(null);
        page.once('dialog', dialogHandler);
        return elementHandle.click().then(() => {
            page.removeListener('dialog', dialogHandler);
            resolve(null);
        }).catch(reject);
    });
}
exports.default = elementClick;
